@@ -32,6 +32,7 @@ from samba.credentials import (
     MUST_USE_KERBEROS,
 )
 from samba._glue import get_burnt_commandline
+import samba
 
 
 def check_bytes(option, opt, value):
@@ -170,6 +171,58 @@ class OptionParser(optparse.OptionParser):
 
         return super().check_values(values, args)
 
+    def format_help(self, formatter=None):
+        # if a magic environment variable is set, we print a docbook
+        # template to add to the samba-tool manpage. Otherwise, we do
+        # normal help.
+        if not os.getenv("SAMBATOOL_HELP_IS_XML"):
+            return super().format_help(formatter=None)
+
+        formatter = SambaDocBookFormatter(1, 9999, 999, True)
+        result = []
+        if self.usage:
+            result.append(self.get_usage() + "\n")
+        if self.description:
+            result.append(self.format_description(formatter) + "\n")
+        result.append("<variablelist>\n")
+        result.append(self.format_option_help(formatter))
+        result.append("</variablelist>\n")
+        result.append(self.format_epilog(formatter))
+        return "".join(result)
+
+
+class SambaDocBookFormatter(optparse.HelpFormatter):
+
+    def _indent(self, s):
+        return '\t' * self.current_indent + s
+
+    def format_usage(self, usage):
+        return self._indent(f"<constant>{usage}</constant>\n")
+
+    def format_heading(self, heading):
+        return self._indent(f"<!--{heading}-->\n")
+
+    def _format_text(self, text):
+        """
+        Format a paragraph of free-form text for inclusion in the
+        help output at the current indentation level.
+        """
+        return self._indent(f"<para>{text}</para>")
+
+    def format_option(self, option):
+        # we don't worry about width (man handles that),
+        # but we try to indent the xml a bit
+        indent = self._indent('  ')
+        result = [indent]
+        result.append('<varlistentry>\n')
+        opts = self.option_strings[option]
+        result.append(f"{indent}  <term>{opts}</term>\n")
+        if option.help:
+            help_text = self.expand_default(option)
+            result.append(f"{indent}  <listitem><para>{help_text}</para></listitem>\n")
+        result.append(f'{indent}</varlistentry>\n')
+        return "".join(result)
+
 
 class OptionGroup(optparse.OptionGroup):
     """Samba OptionGroup base class.
@@ -310,10 +363,9 @@ class VersionOptions(OptionGroup):
         super().__init__(parser, "Version Options")
         self.add_option("-V", "--version", action="callback",
                         callback=self._display_version,
-                        help="Display version number")
+                        help=f"Display version number ({samba.version})")
 
     def _display_version(self, option, opt_str, arg, parser):
-        import samba
         print(samba.version)
         sys.exit(0)
 
@@ -439,6 +491,7 @@ class CredentialsOptions(OptionGroup):
         self.creds.set_bind_dn(arg)
 
     def _set_krb5_ccache(self, option, opt_str, arg, parser):
+        self.ask_for_password = False
         self.creds.set_kerberos_state(MUST_USE_KERBEROS)
         self.creds.set_named_ccache(arg)
 
