@@ -23,13 +23,14 @@ import textwrap
 import traceback
 
 import samba
-from ldb import ERR_INVALID_CREDENTIALS, LdbError
+from ldb import ERR_INVALID_CREDENTIALS, ERR_INSUFFICIENT_ACCESS_RIGHTS, LdbError
 from samba import colour
 from samba.auth import system_session
 from samba.getopt import Option, OptionParser
 from samba.logger import get_samba_logger
 from samba.samdb import SamDB
 from samba.dcerpc.security import SDDLValueError
+from samba import getopt as options
 
 from .encoders import JSONEncoder
 
@@ -82,8 +83,8 @@ class Command(object):
     # synopsis must be defined in all subclasses in order to provide the
     # command usage
     synopsis = None
-    takes_args = []
-    takes_options = []
+    takes_args = ()
+    takes_options = ()
     takes_optiongroups = {}
 
     hidden = False
@@ -93,6 +94,7 @@ class Command(object):
     raw_argv = None
     raw_args = None
     raw_kwargs = None
+    preferred_output_format = None
 
     def _set_files(self, outf=None, errf=None):
         if outf is not None:
@@ -104,10 +106,23 @@ class Command(object):
         self._set_files(outf, errf)
 
     def usage(self, prog=None):
-        parser, _ = self._create_parser(prog)
+        parser, _ = self._create_parser(prog or self.command_name)
         parser.print_usage()
 
     def _print_error(self, msg, evalue=None, klass=None):
+        if self.preferred_output_format == 'json':
+            if evalue is None:
+                evalue = 1
+            else:
+                msg = f"{msg} - {evalue}"
+            if klass is not None:
+                kwargs = {'error class': klass}
+            else:
+                kwargs = {}
+
+            self.print_json_status(evalue, msg, **kwargs)
+            return
+
         err = colour.c_DARK_RED("ERROR")
         klass = '' if klass is None else f'({klass})'
 
@@ -155,6 +170,50 @@ class Command(object):
         json.dump(data, self.outf, cls=JSONEncoder, indent=2, sort_keys=True)
         self.outf.write("\n")
 
+    def print_json_status(self, error=None, message=None, **kwargs):
+        """For commands that really have nothing to say when they succeed
+        (`samba-tool foo delete --json`), we can still emit
+        '{"status": "OK"}\n'. And if they fail they can say:
+        '{"status": "error"}\n'.
+        This function hopes to keep things consistent.
+
+        If error is true-ish but not True, it is stringified and added
+        as a message. For example, if error is an LdbError with an
+        OBJECT_NOT_FOUND code, self.print_json_status(error) results
+        in this:
+
+            '{"status": "error", "message": "object not found"}\n'
+
+        unless an explicit message is added, in which case that is
+        used. A message can be provided on success, like this:
+
+            '{"status": "OK", "message": "thanks for asking!"}\n'
+
+        Extra keywords can be added too.
+
+        In summary, you might go:
+
+            try:
+                samdb.delete(dn)
+            except Exception as e:
+                print_json_status(e)
+                return
+            print_json_status()
+        """
+        data = {}
+        if error:
+            data['status'] = 'error'
+            if error is not True:
+                data['message'] = str(error)
+        else:
+            data['status'] = 'OK'
+
+        if message is not None:
+            data['message'] = message
+
+        data.update(kwargs)
+        self.print_json(data)
+
     def show_command_error(self, e):
         """display a command error"""
         if isinstance(e, CommandError):
@@ -184,6 +243,12 @@ class Command(object):
             elif ldb_emsg.startswith("Unable to open tdb "):
                 self._print_error(message, ldb_emsg, 'ldb')
                 force_traceback = False
+            elif ldb_ecode == ERR_INSUFFICIENT_ACCESS_RIGHTS:
+                self._print_error("User has insufficient access rights")
+                force_traceback = False
+            elif ldb_emsg == "Operation unavailable without authentication":
+                self._print_error(ldb_emsg)
+                force_traceback = False
             else:
                 self._print_error(message, ldb_emsg, 'ldb')
 
@@ -217,10 +282,16 @@ class Command(object):
             option_class=Option)
         parser.add_options(self.takes_options)
         optiongroups = {}
+
+        # let samba-tool subcommands process --version
+        if "versionopts" not in self.takes_optiongroups:
+            self.takes_optiongroups["_versionopts"] = options.VersionOptions
+
         for name in sorted(self.takes_optiongroups.keys()):
             optiongroup = self.takes_optiongroups[name]
             optiongroups[name] = optiongroup(parser)
             parser.add_option_group(optiongroups[name])
+
         if self.use_colour:
             parser.add_option("--color",
                               help="use colour if available (default: auto)",
@@ -249,12 +320,43 @@ class Command(object):
             return -1
 
         # Filter out options from option groups
+        #
+        # run() methods on subclasses receive all direct options as
+        # keyword arguments, but options that come from OptionGroups
+        # (for example, --configfile from SambaOpts group) are removed
+        # from the direct keyword arguments list, and the option group
+        # itself becomes a keyword argument. The option can be
+        # accessed as an attribute of that (e.g. sambaopts.configfile).
+        #
+        # This allows for option groups to grow without needing to
+        # change the signature for all samba-tool tools.
+        #
+        # _versionopts special case.
+        # ==========================
+        #
+        # The _versionopts group was added automatically, and is
+        # removed here. It only has the -V/--version option, and that
+        # will have triggered already if given (as will --help, and
+        # errors on unknown options).
+        #
+        # Some subclasses take 'versionopts' which they expect to
+        # receive but never use.
+
         kwargs = dict(opts.__dict__)
-        for option_group in parser.option_groups:
+
+        for og_name, option_group in optiongroups.items():
             for option in option_group.option_list:
                 if option.dest is not None and option.dest in kwargs:
                     del kwargs[option.dest]
-        kwargs.update(optiongroups)
+            if og_name != '_versionopts':
+                kwargs[og_name] = option_group
+
+        if kwargs.get('output_format') == 'json':
+            self.preferred_output_format = 'json'
+        else:
+            # we need to reset this for the tests that reuse the
+            # samba-tool object.
+            self.preferred_output_format = None
 
         if self.use_colour:
             self.apply_colour_choice(kwargs.pop('color', 'auto'))
@@ -344,7 +446,10 @@ class SuperCommand(Command):
                 sub = self.subcommands[a]
                 return sub._resolve(sub_path, *sub_args, outf=outf, errf=errf)
 
-            elif a in ['--help', 'help', None, '-h', '-V', '--version']:
+            if a in ['-V', '--version']:
+                return (self, [a])
+
+            if a in ['--help', 'help', None, '-h']:
                 # we pass these to the leaf node.
                 if a == 'help':
                     a = '--help'
@@ -355,15 +460,16 @@ class SuperCommand(Command):
             print("%s: no such subcommand: %s\n" % (path, a), file=self.outf)
             return (self, [])
 
-        # We didn't find a subcommand, but maybe we found e.g. --version
-        print("%s: missing subcommand\n" % (path), file=self.outf)
+        # We didn't find a subcommand, but maybe we found e.g. --help
+        if not deferred_args:
+            print("%s: missing subcommand\n" % (path), file=self.outf)
         return (self, deferred_args)
 
     def _run(self, *argv):
         epilog = "\nAvailable subcommands:\n"
 
         subcmds = sorted(self.subcommands.keys())
-        max_length = max([len(c) for c in subcmds])
+        max_length = max([len(c) for c in subcmds], default=0)
         for cmd_name in subcmds:
             cmd = self.subcommands[cmd_name]
             if cmd.hidden:
@@ -371,7 +477,7 @@ class SuperCommand(Command):
             epilog += "  %*s  - %s\n" % (
                 -max_length, cmd_name, cmd.short_description)
 
-        epilog += ("For more help on a specific subcommand, please type: "
+        epilog += ("\nFor more help on a specific subcommand, please type: "
                    f"{self.command_name} <subcommand> (-h|--help)\n")
 
         parser, optiongroups = self._create_parser(self.command_name, epilog=epilog)
