@@ -31,7 +31,7 @@ import ctypes
 class NetJoinNoSpnegoTests(samba.tests.TestCaseInTempDir):
 
     def setUp(self):
-        super(NetJoinNoSpnegoTests, self).setUp()
+        super().setUp()
         self.domain = os.environ["DOMAIN"]
         self.server = os.environ["SERVER"]
         self.lp = self.get_loadparm()
@@ -39,9 +39,6 @@ class NetJoinNoSpnegoTests(samba.tests.TestCaseInTempDir):
         self.lp.set("lock dir", self.tempdir)
         self.lp.set("state directory", self.tempdir)
 
-    def tearDown(self):
-        super(NetJoinNoSpnegoTests, self).tearDown()
-
     def test_net_join_no_spnego(self):
         self.lp.set("client ipc max protocol", "NT1")
         self.lp.set("client use spnego", "no")
