@@ -15,15 +15,17 @@
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #
 
-import os, ldb
+import os
+import ldb
 from samba.tests.samba_tool.base import SambaToolCmdTest
 
+
 class JoinCmdTestCase(SambaToolCmdTest):
-    """Test for samba-tool fsmo show subcommand"""
+    """Test for samba-tool domain join subcommand"""
 
     def test_rejoin(self):
         """Run domain join to confirm it errors because we are already joined"""
         (result, out, err) = self.runsubcmd("domain", "join", os.environ["REALM"], "dc", "-U%s%%%s" % (os.environ["USERNAME"], os.environ["PASSWORD"]))
 
         self.assertCmdFail(result)
-        self.assertTrue("Not removing account" in err,"Should fail with exception")
+        self.assertTrue("Not removing account" in err, "Should fail with exception")
